/*
Functions described in the JavaScript API documentation (http://www.spicynodes.org/developers-js.html) and used in this mashup: 
- getTreeobject	::			spicynodesMovie.sn_call('getTreeObject,true,true');	
- nodeId		::			spicynodesMovie.sn_call("nodeid,"+ node.id);
- appendNode	::			spicynodesMovie.sn_call('appendnode,' + currentSelected.id+ "," + newNode);  
- getTreeXML	::			spicynodesMovie.sn_call('getTreeXML,true,true');  

*/

/*Google map*/
var map;
		  
/*Array for city selection combobox values*/
var citiesList;

/*Called from <body> onLoad event. Initializes Google Maps with some custom parameters. */
function initializeMaps() {	
	var initialLatlng = new google.maps.LatLng(-34.397, 145.644);
	var myOptions = {
	  zoom: 8,
	  center: initialLatlng,
	  mapTypeId: google.maps.MapTypeId.HYBRID
	};

	// Geocoder used to know current location
	geocoder = new google.maps.Geocoder();

	// map is a global variable
	map = new google.maps.Map(document.getElementById("map_canvas"), myOptions);

	// listen to current location, to update fields below google map
	google.maps.event.addListener(map, 'dragend', function() {
		updateCurrentLocation(map.getCenter());
	});
}


function updateCurrentLocation(curLoc){
	if (geocoder) {
		var latlngStr = String(curLoc);
		latlngStr = latlngStr.substring(1,latlngStr.length-1);
		latlngStr = latlngStr.split(",");
		var lat = parseFloat(latlngStr[0]);
		var lng = parseFloat(latlngStr[1]);
		var latlng = new google.maps.LatLng(lat, lng);
		  geocoder.geocode({'latLng': latlng}, function(results, status) {
			if (status == google.maps.GeocoderStatus.OK) {
				document.forms['htmlForm'].elements['locationName'].value = results[1].formatted_address;
			} else {
				document.forms['htmlForm'].elements['locationName'].value = "";
			}
		  });
		}
		document.forms['htmlForm'].elements['latitudeValue'].value = lat;
		document.forms['htmlForm'].elements['longitudeValue'].value = lng;
}

function onSNAnimationStart(node)
{			
	// update the currentSelected node
	currentSelected = node;	
	if(node) {
		// start 
		spicynodesMovie.sn_call('getTreeObject,true,true');	
	}
}


/* handler for values returning from SpicyNodes after calling a SpicyNodes method */
function callBackFromSN(methodName, returningValue)
{
	//alert("callBackFromSN() methodName: " + methodName + "  returningValue: " + returningValue);
	
	switch(methodName.toLowerCase())
	{
		case "gettreeobject": 	{
									updateCityCombobox(returningValue);
									goToCity(currentSelected.id);
									break;
								}
		case "gettreexml": 		{
									document.htmlForm.xmlOutput.value = returningValue;
									break;
								}	
			
	}
}

//////////////// POPULATIONG COMBOBOX
function updateCityCombobox(tree)
{
	citiesList = getIndexesList(tree);
	var optionsLen = document.htmlForm.cityCombo.options.length;
	
	// remove all combobox items
	for(m = document.htmlForm.cityCombo.options.length - 1; m > 0 ; m-- ) {
		document.htmlForm.cityCombo.options[m] = null;		
	}	
	
	var listLen = citiesList.length;
	
	// populate combobox with new items, ommiting the home node (i = 1 instead of i = 0)
	for(var i = 1; i < listLen; i++) {		
		document.htmlForm.cityCombo.options[i] = new Option(citiesList[i].data.id + ": " + citiesList[i].label, citiesList[i].data.id);
	}
}
function getIndexesList(tree) {
	var list = [];
	getIndexesListRec(tree, list);	
	
	return list;	
}
function getIndexesListRec(node, list) {
	list.push({label:node.label, data:node});	
	
	var children = node.children;	
	
	for(var k in children) {
		getIndexesListRec(children[k],list);
	}
}
///////////////// END OF COMBOBOX OPERATIONS



// Changes the city - called either by selecting a node in the layout or by selecting a combobox item.
function goToCity(nodeId)
{
	var node;
	
	var len = citiesList.length;
	
	// select the city data 
	for(var k = 1; k < len; k++) {		
		if(nodeId == citiesList[k].data.id) {
			node = citiesList[k].data;
			break;
		}
	}		
	
	var lat = node.lat;
	var lng = node.lng;
	
	if(!lat || !lng) {		
		alert("goToCity() no latitude or longitude defined for node: " + node.label);
		
		changeCurrentNode(node);
		return;
	}
	
	// get a new Google Maps latitude-longitude object
	var newLatLng = new google.maps.LatLng( Number(node.lat), Number(node.lng) );	
	
	// center map above selected city
	map.panTo(newLatLng);
	map.setZoom( Number(node.zoom) );
	
	// place the new marker
	placeNewMarker(newLatLng, node.label);
	
	// if not yet selected, select the node in SpicyNodes layout (this is needed when a city is selected from combobox)
	if(currentSelected.id != node.id) {
		spicynodesMovie.sn_call("nodeid,"+ node.id);
	}
}

/* Places a new marker. The new marker, when clicked pop an info window. */
function placeNewMarker(latLng, cityName)
{
	var newInfoWindow = new google.maps.InfoWindow({
			content: "You have just clicked the '" + cityName + "' marker..."
	});
				
	var newMarker = new google.maps.Marker({
		position: latLng, 
		map: map,
		title: cityName + " marker."
	});   

	// define the function triggered after clicking a marker
	google.maps.event.addListener(newMarker, 'click', function() {		
		newInfoWindow.open(map, newMarker);
	});

}



/* Called by pressing the "Add" button of the HTML form. Creates a new XML node that is added to current SpicyNodes nodemap.*/
function appendNode(locationName, description, latitude, longitude, zoom)
{	
   var newNode;         
   newNode = "<node label='" + locationName + "' lat='" + latitude + "' lng='" + longitude + "' zoom='" + zoom + "'  >"
   if(description.length > 0) {
	   newNode += "<![CDATA[" + description + "]]>"
   }
   newNode += "</node>";
   
   /* append a new child to currently selected node */
   spicynodesMovie.sn_call('appendnode,' + currentSelected.id+ "," + newNode);  
}




/* Fills the HTML text area with the currently displayed XML nodemap (including any nodes added by appendNode() ) */
function getXML()
{
	spicynodesMovie.sn_call('getTreeXML,true,true');  
}